@@ -25,7 +25,7 @@
 
 /******************************************************************************/
 
-(function() {
+(( ) => {
 
 /******************************************************************************/
 
@@ -58,14 +58,13 @@ window.addEventListener('beforeunload', ( ) => {
     );
 });
 
-
 /******************************************************************************/
 
 // This is to give a visual hint that the content of user blacklist has changed.
 
 const userFiltersChanged = function(changed) {
     if ( typeof changed !== 'boolean' ) {
-        changed = cmEditor.getValue().trim() !== cachedUserFilters;
+        changed = self.hasUnsavedData();
     }
     uDom.nodeFromId('userFiltersApply').disabled = !changed;
     uDom.nodeFromId('userFiltersRevert').disabled = !changed;
@@ -101,13 +100,6 @@ const renderUserFilters = function(first) {
 
 /******************************************************************************/
 
-const allFiltersApplyHandler = function() {
-    messaging.send('dashboard', { what: 'reloadAllFilters' });
-    uDom('#userFiltersApply').prop('disabled', true );
-};
-
-/******************************************************************************/
-
 const handleImportFilePicker = function() {
     // https://github.com/chrisaljoudi/uBlock/issues/1004
     // Support extraction of filters from ABP backup file
@@ -188,7 +180,8 @@ const applyChanges = function() {
         details => {
             if ( details.error ) { return; }
             cachedUserFilters = details.content.trim();
-            allFiltersApplyHandler();
+            userFiltersChanged(false);
+            messaging.send('dashboard', { what: 'reloadAllFilters' });
         }
     );
 };
@@ -220,6 +213,12 @@ self.cloud.onPull = setCloudData;
 
 /******************************************************************************/
 
+self.hasUnsavedData = function() {
+    return cmEditor.getValue().trim() !== cachedUserFilters;
+};
+
+/******************************************************************************/
+
 // Handle user interaction
 uDom('#importUserFiltersFromFile').on('click', startImportFilePicker);
 uDom('#importFilePicker').on('change', handleImportFilePicker);
