@@ -23,7 +23,7 @@
 
 /******************************************************************************/
 
-µBlock.assets = (function() {
+µBlock.assets = (( ) => {
 
 /******************************************************************************/
 
@@ -36,7 +36,7 @@ const api = {};
 
 /******************************************************************************/
 
-var observers = [];
+const observers = [];
 
 api.addObserver = function(observer) {
     if ( observers.indexOf(observer) === -1 ) {
@@ -45,16 +45,16 @@ api.addObserver = function(observer) {
 };
 
 api.removeObserver = function(observer) {
-    var pos;
+    let pos;
     while ( (pos = observers.indexOf(observer)) !== -1 ) {
         observers.splice(pos, 1);
     }
 };
 
-var fireNotification = function(topic, details) {
-    var result, r;
-    for ( var i = 0; i < observers.length; i++ ) {
-        r = observers[i](topic, details);
+const fireNotification = function(topic, details) {
+    let result;
+    for ( const observer of observers ) {
+        const r = observer(topic, details);
         if ( r !== undefined ) { result = r; }
     }
     return result;
@@ -62,28 +62,8 @@ var fireNotification = function(topic, details) {
 
 /******************************************************************************/
 
-api.fetchText = function(url, onLoad, onError) {
-    const isExternal = reIsExternalPath.test(url);
-    let actualUrl = isExternal ? url : vAPI.getURL(url);
-
-    // https://github.com/gorhill/uBlock/issues/2592
-    //   Force browser cache to be bypassed, but only for resources which have
-    //   been fetched more than one hour ago.
-    if ( isExternal ) {
-        const queryValue = `_=${Math.floor(Date.now() / 3600000) % 12}`;
-        if ( actualUrl.indexOf('?') === -1 ) {
-            actualUrl += '?';
-        } else {
-            actualUrl += '&';
-        }
-        actualUrl += queryValue;
-    }
-
-    if ( typeof onError !== 'function' ) {
-        onError = onLoad;
-    }
-
-    return new Promise(resolve => {
+api.fetch = function(url, options = {}) {
+    return new Promise((resolve, reject) => {
     // Start of executor
 
     const timeoutAfter = µBlock.hiddenSettings.assetFetchTimeout * 1000 || 30000;
@@ -102,20 +82,6 @@ api.fetchText = function(url, onLoad, onError) {
         }
     };
 
-    const onResolve = function(details) {
-        if ( onLoad instanceof Function ) {
-            return onLoad(details);
-        }
-        resolve(details);
-    };
-
-    const onReject = function(details) {
-        if ( onError instanceof Function ) {
-            return onError(details);
-        }
-        resolve(details);
-    };
-
     // https://github.com/gorhill/uMatrix/issues/15
     const onLoadEvent = function() {
         cleanup();
@@ -127,21 +93,10 @@ api.fetchText = function(url, onLoad, onError) {
             statusText: this.statusText || ''
         };
         if ( details.statusCode < 200 || details.statusCode >= 300 ) {
-            return onReject(details);
+            return reject(details);
         }
-        // consider an empty result to be an error
-        if ( stringIsNotEmpty(this.responseText) === false ) {
-            return onReject(details);
-        }
-        // we never download anything else than plain text: discard if response
-        // appears to be a HTML document: could happen when server serves
-        // some kind of error page I suppose
-        const text = this.responseText.trim();
-        if ( text.startsWith('<') && text.endsWith('>') ) {
-            return onReject(details);
-        }
-        details.content = this.responseText;
-        onResolve(details);
+        details.content = this.response;
+        resolve(details);
     };
 
     const onErrorEvent = function() {
@@ -149,9 +104,9 @@ api.fetchText = function(url, onLoad, onError) {
         µBlock.logger.writeOne({
             realm: 'message',
             type: 'error',
-            text: errorCantConnectTo.replace('{{msg}}', actualUrl)
+            text: errorCantConnectTo.replace('{{msg}}', url)
         });
-        onReject({ url, content: '' });
+        reject({ url, content: '' });
     };
 
     const onTimeout = function() {
@@ -173,12 +128,12 @@ api.fetchText = function(url, onLoad, onError) {
     // I am pretty sure it used to work, but now using a URL such as
     // `file:///` on Chromium 40 results in an exception being thrown.
     try {
-        xhr.open('get', actualUrl, true);
+        xhr.open('get', url, true);
         xhr.addEventListener('load', onLoadEvent);
         xhr.addEventListener('error', onErrorEvent);
         xhr.addEventListener('abort', onErrorEvent);
         xhr.addEventListener('progress', onProgressEvent);
-        xhr.responseType = 'text';
+        xhr.responseType = options.responseType || 'text';
         xhr.send();
         timeoutTimer = vAPI.setTimeout(onTimeout, timeoutAfter);
     } catch (e) {
@@ -191,6 +146,70 @@ api.fetchText = function(url, onLoad, onError) {
 
 /******************************************************************************/
 
+api.fetchText = function(url, onLoad, onError) {
+    const isExternal = reIsExternalPath.test(url);
+    let actualUrl = isExternal ? url : vAPI.getURL(url);
+
+    // https://github.com/gorhill/uBlock/issues/2592
+    //   Force browser cache to be bypassed, but only for resources which have
+    //   been fetched more than one hour ago.
+    //
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/682#issuecomment-515197130
+    //   Provide filter list authors a way to completely bypass
+    //   the browser cache.
+    if ( isExternal ) {
+        const cacheBypassToken =
+            µBlock.hiddenSettings.updateAssetBypassBrowserCache
+                ? Math.floor(Date.now() /    1000) % 86400
+                : Math.floor(Date.now() / 3600000) %    12;
+        const queryValue = `_=${cacheBypassToken}`;
+        if ( actualUrl.indexOf('?') === -1 ) {
+            actualUrl += '?';
+        } else {
+            actualUrl += '&';
+        }
+        actualUrl += queryValue;
+    }
+
+    if ( typeof onError !== 'function' ) {
+        onError = onLoad;
+    }
+
+    const onResolve = function(details) {
+        if ( onLoad instanceof Function ) {
+            return onLoad(details);
+        }
+        return details;
+    };
+
+    const onReject = function(details) {
+        details.content = '';
+        if ( onError instanceof Function ) {
+            return onError(details);
+        }
+        return details;
+    };
+
+    return api.fetch(url).then(details => {
+        // consider an empty result to be an error
+        if ( stringIsNotEmpty(details.content) === false ) {
+            return onReject(details);
+        }
+        // we never download anything else than plain text: discard if response
+        // appears to be a HTML document: could happen when server serves
+        // some kind of error page I suppose
+        const text = details.content.trim();
+        if ( text.startsWith('<') && text.endsWith('>') ) {
+            return onReject(details);
+        }
+        return onResolve(details);
+    }).catch(details => {
+        return onReject(details);
+    });
+};
+
+/******************************************************************************/
+
 // https://github.com/gorhill/uBlock/issues/3331
 //   Support the seamless loading of sublists.
 
@@ -414,7 +433,7 @@ const updateAssetSourceRegistry = function(json, silent) {
     saveAssetSourceRegistry();
 };
 
-const getAssetSourceRegistry = function(callback) {
+const getAssetSourceRegistry = function() {
     if ( assetSourceRegistryPromise === undefined ) {
         assetSourceRegistryPromise = µBlock.cacheStorage.get(
             'assetSourceRegistry'
@@ -424,34 +443,33 @@ const getAssetSourceRegistry = function(callback) {
                 bin.assetSourceRegistry instanceof Object
             ) {
                 assetSourceRegistry = bin.assetSourceRegistry;
-                return;
+                return assetSourceRegistry;
             }
-            return new Promise(resolve => {
-                api.fetchText(
-                    µBlock.assetsBootstrapLocation || 'assets/assets.json',
-                    details => {
-                        updateAssetSourceRegistry(details.content, true);
-                        resolve();
-                    }
-                );
+            return api.fetchText(
+                µBlock.assetsBootstrapLocation || 'assets/assets.json'
+            ).then(details => {
+                return details.content !== ''
+                    ? details
+                    : api.fetchText('assets/assets.json');
+            }).then(details => {
+                updateAssetSourceRegistry(details.content, true);
+                return assetSourceRegistry;
             });
         });
     }
 
-    assetSourceRegistryPromise.then(( ) => {
-        callback(assetSourceRegistry);
-    });
+    return assetSourceRegistryPromise;
 };
 
 api.registerAssetSource = function(assetKey, details) {
-    getAssetSourceRegistry(function() {
+    getAssetSourceRegistry().then(( ) => {
         registerAssetSource(assetKey, details);
         saveAssetSourceRegistry(true);
     });
 };
 
 api.unregisterAssetSource = function(assetKey) {
-    getAssetSourceRegistry(function() {
+    getAssetSourceRegistry().then(( ) => {
         unregisterAssetSource(assetKey);
         saveAssetSourceRegistry(true);
     });
@@ -693,6 +711,10 @@ api.get = function(assetKey, options, callback) {
     } else if ( typeof callback !== 'function' ) {
         callback = noopfunc;
     }
+    // This can happen if the method was called as a thenable.
+    if ( options instanceof Object === false ) {
+        options = {};
+    }
 
     return new Promise(resolve => {
     // start of executor
@@ -708,18 +730,30 @@ api.get = function(assetKey, options, callback) {
         contentURLs,
         contentURL;
 
-    const reportBack = function(content, err) {
-        const details = { assetKey: assetKey, content: content };
+    const reportBack = (content, err) => {
+        const details = { assetKey, content };
         if ( err ) {
             details.error = assetDetails.lastError = err;
         } else {
             assetDetails.lastError = undefined;
         }
+        if ( options.needSourceURL ) {
+            if (
+                contentURL === undefined &&
+                assetCacheRegistry instanceof Object &&
+                assetCacheRegistry[assetKey] instanceof Object
+            ) {
+                details.sourceURL = assetCacheRegistry[assetKey].remoteURL;
+            }
+            if ( reIsExternalPath.test(contentURL) ) {
+                details.sourceURL = contentURL;
+            }
+        }
         callback(details);
         resolve(details);
     };
 
-    const onContentNotLoaded = function() {
+    const onContentNotLoaded = ( ) => {
         let isExternal;
         while ( (contentURL = contentURLs.shift()) ) {
             isExternal = reIsExternalPath.test(contentURL);
@@ -737,7 +771,7 @@ api.get = function(assetKey, options, callback) {
         }
     };
 
-    const onContentLoaded = function(details) {
+    const onContentLoaded = details => {
         if ( stringIsNotEmpty(details.content) === false ) {
             onContentNotLoaded();
             return;
@@ -751,11 +785,11 @@ api.get = function(assetKey, options, callback) {
         reportBack(details.content);
     };
 
-    const onCachedContentLoaded = function(details) {
+    const onCachedContentLoaded = details => {
         if ( details.content !== '' ) {
             return reportBack(details.content);
         }
-        getAssetSourceRegistry(function(registry) {
+        getAssetSourceRegistry().then(registry => {
             assetDetails = registry[assetKey] || {};
             if ( typeof assetDetails.contentURL === 'string' ) {
                 contentURLs = [ assetDetails.contentURL ];
@@ -776,12 +810,12 @@ api.get = function(assetKey, options, callback) {
 /******************************************************************************/
 
 const getRemote = function(assetKey, callback) {
-   var assetDetails = {},
-        contentURLs,
-        contentURL;
+    let assetDetails = {};
+    let contentURLs;
+    let contentURL;
 
-    var reportBack = function(content, err) {
-        var details = { assetKey: assetKey, content: content };
+    const reportBack = function(content, err) {
+        const details = { assetKey: assetKey, content: content };
         if ( err ) {
             details.error = assetDetails.lastError = err;
         } else {
@@ -790,7 +824,7 @@ const getRemote = function(assetKey, callback) {
         callback(details);
     };
 
-    var onRemoteContentLoaded = function(details) {
+    const onRemoteContentLoaded = function(details) {
         if ( stringIsNotEmpty(details.content) === false ) {
             registerAssetSource(assetKey, { error: { time: Date.now(), error: 'No content' } });
             tryLoading();
@@ -804,8 +838,8 @@ const getRemote = function(assetKey, callback) {
         reportBack(details.content);
     };
 
-    var onRemoteContentError = function(details) {
-        var text = details.statusText;
+    const onRemoteContentError = function(details) {
+        let text = details.statusText;
         if ( details.statusCode === 0 ) {
             text = 'network error';
         }
@@ -813,7 +847,7 @@ const getRemote = function(assetKey, callback) {
         tryLoading();
     };
 
-    var tryLoading = function() {
+    const tryLoading = function() {
         while ( (contentURL = contentURLs.shift()) ) {
             if ( reIsExternalPath.test(contentURL) ) { break; }
         }
@@ -827,7 +861,7 @@ const getRemote = function(assetKey, callback) {
         }
     };
 
-    getAssetSourceRegistry(function(registry) {
+    getAssetSourceRegistry().then(registry => {
         assetDetails = registry[assetKey] || {};
         if ( typeof assetDetails.contentURL === 'string' ) {
             contentURLs = [ assetDetails.contentURL ];
@@ -861,21 +895,18 @@ api.put = function(assetKey, content, callback) {
 /******************************************************************************/
 
 api.metadata = function(callback) {
-    var assetRegistryReady = false,
-        cacheRegistryReady = false;
-
-    var onReady = function() {
-        var assetDict = JSON.parse(JSON.stringify(assetSourceRegistry)),
-            cacheDict = assetCacheRegistry,
-            assetEntry, cacheEntry,
-            now = Date.now(), obsoleteAfter;
-        for ( var assetKey in assetDict ) {
-            assetEntry = assetDict[assetKey];
-            cacheEntry = cacheDict[assetKey];
+    const onReady = function() {
+        const assetDict = JSON.parse(JSON.stringify(assetSourceRegistry));
+        const cacheDict = assetCacheRegistry;
+        const now = Date.now();
+        for ( const assetKey in assetDict ) {
+            const assetEntry = assetDict[assetKey];
+            const cacheEntry = cacheDict[assetKey];
             if ( cacheEntry ) {
                 assetEntry.cached = true;
                 assetEntry.writeTime = cacheEntry.writeTime;
-                obsoleteAfter = cacheEntry.writeTime + assetEntry.updateAfter * 86400000;
+                const obsoleteAfter =
+                    cacheEntry.writeTime + assetEntry.updateAfter * 86400000;
                 assetEntry.obsolete = obsoleteAfter < now;
                 assetEntry.remoteURL = cacheEntry.remoteURL;
             } else if (
@@ -883,22 +914,18 @@ api.metadata = function(callback) {
                 assetEntry.contentURL.length !== 0
             ) {
                 assetEntry.writeTime = 0;
-                obsoleteAfter = 0;
                 assetEntry.obsolete = true;
             }
         }
         callback(assetDict);
     };
 
-    getAssetSourceRegistry(function() {
-        assetRegistryReady = true;
-        if ( cacheRegistryReady ) { onReady(); }
-    });
-
-    getAssetCacheRegistry().then(( ) => {
-        cacheRegistryReady = true;
-        if ( assetRegistryReady ) { onReady(); }
-    });
+    Promise.all([
+        getAssetSourceRegistry(),
+        getAssetCacheRegistry(),
+    ]).then(
+        ( ) => onReady()
+    );
 };
 
 /******************************************************************************/
@@ -916,58 +943,44 @@ api.rmrf = function() {
 /******************************************************************************/
 
 // Asset updater area.
-var updaterStatus,
-    updaterTimer,
-    updaterAssetDelayDefault = 120000,
-    updaterAssetDelay = updaterAssetDelayDefault,
-    updaterUpdated = [],
-    updaterFetched = new Set(),
-    noRemoteResources;
+const updaterAssetDelayDefault = 120000;
+const updaterUpdated = [];
+const updaterFetched = new Set();
 
-var updateFirst = function() {
-    // https://github.com/gorhill/uBlock/commit/126110c9a0a0630cd556f5cb215422296a961029
-    //   Firefox extension reviewers do not want uBO/webext to fetch its own
-    //   scriptlets/resources asset from the project's own repo (github.com).
-    // https://github.com/uBlockOrigin/uAssets/issues/1647#issuecomment-371456830
-    //   Allow self-hosted dev build to update: if update_url is present but
-    //   null, assume the extension is hosted on AMO.
-    if ( noRemoteResources === undefined ) {
-        noRemoteResources =
-            vAPI.webextFlavor.soup.has('firefox') &&
-            vAPI.webextFlavor.soup.has('webext') &&
-            vAPI.webextFlavor.soup.has('devbuild') === false;
-    }
+let updaterStatus,
+    updaterTimer,
+    updaterAssetDelay = updaterAssetDelayDefault;
+
+const updateFirst = function() {
     updaterStatus = 'updating';
     updaterFetched.clear();
-    updaterUpdated = [];
+    updaterUpdated.length = 0;
     fireNotification('before-assets-updated');
     updateNext();
 };
 
-var updateNext = function() {
-    var assetDict, cacheDict;
+const updateNext = function() {
+    let assetDict, cacheDict;
 
     // This will remove a cached asset when it's no longer in use.
-    var garbageCollectOne = function(assetKey) {
-        var cacheEntry = cacheDict[assetKey];
+    const garbageCollectOne = function(assetKey) {
+        const cacheEntry = cacheDict[assetKey];
         if ( cacheEntry && cacheEntry.readTime < assetCacheRegistryStartTime ) {
             assetCacheRemove(assetKey);
         }
     };
 
-    var findOne = function() {
-        var now = Date.now(),
-            assetEntry, cacheEntry;
-        for ( var assetKey in assetDict ) {
-            assetEntry = assetDict[assetKey];
+    const findOne = function() {
+        const now = Date.now();
+        for ( const assetKey in assetDict ) {
+            const assetEntry = assetDict[assetKey];
             if ( assetEntry.hasRemoteURL !== true ) { continue; }
             if ( updaterFetched.has(assetKey) ) { continue; }
-            cacheEntry = cacheDict[assetKey];
-            if ( cacheEntry && (cacheEntry.writeTime + assetEntry.updateAfter * 86400000) > now ) {
-                continue;
-            }
-            // Update of user scripts/resources forbidden?
-            if ( assetKey === 'ublock-resources' && noRemoteResources ) {
+            const cacheEntry = cacheDict[assetKey];
+            if (
+                cacheEntry &&
+                (cacheEntry.writeTime + assetEntry.updateAfter * 86400000) > now
+            ) {
                 continue;
             }
             if (
@@ -982,7 +995,7 @@ var updateNext = function() {
         }
     };
 
-    var updatedOne = function(details) {
+    const updatedOne = function(details) {
         if ( details.content !== '' ) {
             updaterUpdated.push(details.assetKey);
             if ( details.assetKey === 'assets.json' ) {
@@ -998,8 +1011,8 @@ var updateNext = function() {
         }
     };
 
-    var updateOne = function() {
-        var assetKey = findOne();
+    const updateOne = function() {
+        const assetKey = findOne();
         if ( assetKey === undefined ) {
             return updateDone();
         }
@@ -1007,33 +1020,30 @@ var updateNext = function() {
         getRemote(assetKey, updatedOne);
     };
 
-    getAssetSourceRegistry(function(dict) {
-        assetDict = dict;
-        if ( !cacheDict ) { return; }
-        updateOne();
-    });
-
-    getAssetCacheRegistry().then(dict => {
-        cacheDict = dict;
-        if ( !assetDict ) { return; }
+    Promise.all([
+        getAssetSourceRegistry(),
+        getAssetCacheRegistry(),
+    ]).then(results => {
+        assetDict = results[0];
+        cacheDict = results[1];
         updateOne();
     });
 };
 
-var updateDone = function() {
-    var assetKeys = updaterUpdated.slice(0);
+const updateDone = function() {
+    const assetKeys = updaterUpdated.slice(0);
     updaterFetched.clear();
-    updaterUpdated = [];
+    updaterUpdated.length = 0;
     updaterStatus = undefined;
     updaterAssetDelay = updaterAssetDelayDefault;
     fireNotification('after-assets-updated', { assetKeys: assetKeys });
 };
 
 api.updateStart = function(details) {
-    var oldUpdateDelay = updaterAssetDelay,
-        newUpdateDelay = typeof details.delay === 'number' ?
-            details.delay :
-            updaterAssetDelayDefault;
+    const oldUpdateDelay = updaterAssetDelay;
+    const newUpdateDelay = typeof details.delay === 'number' ?
+        details.delay :
+        updaterAssetDelayDefault;
     updaterAssetDelay = Math.min(oldUpdateDelay, newUpdateDelay);
     if ( updaterStatus !== undefined ) {
         if ( newUpdateDelay < oldUpdateDelay ) {
@@ -1055,6 +1065,11 @@ api.updateStop = function() {
     }
 };
 
+api.isUpdating = function() {
+    return updaterStatus === 'updating' &&
+           updaterAssetDelay <= µBlock.hiddenSettings.manualUpdateAssetFetchPeriod;
+};
+
 /******************************************************************************/
 
 return api;
