@@ -197,6 +197,79 @@ vAPI.SafeAnimationFrame.prototype = {
 /******************************************************************************/
 /******************************************************************************/
 
+// https://github.com/uBlockOrigin/uBlock-issues/issues/552
+//   Listen and report CSP violations so that blocked resources through CSP
+//   are properly reported in the logger.
+
+{
+    const newEvents = new Set();
+    const allEvents = new Set();
+    let timer;
+
+    const send = function() {
+        vAPI.messaging.send(
+            'scriptlets',
+            {
+                what: 'securityPolicyViolation',
+                type: 'net',
+                docURL: document.location.href,
+                violations: Array.from(newEvents),
+            },
+            response => {
+                if ( response === true ) { return; }
+                stop();
+            }
+        );
+        for ( const event of newEvents ) {
+            allEvents.add(event);
+        }
+        newEvents.clear();
+    };
+
+    const sendAsync = function() {
+        if ( timer !== undefined ) { return; }
+        timer = self.requestIdleCallback(
+            ( ) => { timer = undefined; send(); },
+            { timeout: 2000 }
+        );
+    };
+
+    const listener = function(ev) {
+        if ( ev.isTrusted !== true ) { return; }
+        if ( ev.disposition !== 'enforce' ) { return; }
+        const json = JSON.stringify({
+            url: ev.blockedURL || ev.blockedURI,
+            policy: ev.originalPolicy,
+            directive: ev.effectiveDirective || ev.violatedDirective,
+        });
+        if ( allEvents.has(json) ) { return; }
+        newEvents.add(json);
+        sendAsync();
+    };
+
+    const stop = function() {
+        newEvents.clear();
+        allEvents.clear();
+        if ( timer !== undefined ) {
+            self.cancelIdleCallback(timer);
+            timer = undefined;
+        }
+        document.removeEventListener('securitypolicyviolation', listener);
+        vAPI.shutdown.remove(stop);
+    };
+
+    document.addEventListener('securitypolicyviolation', listener);
+    vAPI.shutdown.add(stop);
+
+    // We need to call at least once to find out whether we really need to
+    // listen to CSP violations.
+    sendAsync();
+}
+
+/******************************************************************************/
+/******************************************************************************/
+/******************************************************************************/
+
 vAPI.domWatcher = (function() {
 
     const addedNodeLists = [];
@@ -343,7 +416,7 @@ vAPI.domWatcher = (function() {
 /******************************************************************************/
 /******************************************************************************/
 
-vAPI.matchesProp = (function() {
+vAPI.matchesProp = (( ) => {
     const docElem = document.documentElement;
     if ( typeof docElem.matches !== 'function' ) {
         if ( typeof docElem.mozMatchesSelector === 'function' ) {
@@ -403,14 +476,10 @@ vAPI.DOMFilterer = (function() {
             }
             this.needle = new RegExp(arg0, arg1);
         }
-        exec(input) {
-            const output = [];
-            for ( const node of input ) {
-                if ( this.needle.test(node.textContent) ) {
-                    output.push(node);
-                }
+        transpose(node, output) {
+            if ( this.needle.test(node.textContent) ) {
+                output.push(node);
             }
-            return output;
         }
     };
 
@@ -418,24 +487,17 @@ vAPI.DOMFilterer = (function() {
         constructor(task) {
             this.pselector = new PSelector(task[1]);
         }
-        exec(input) {
-            const output = [];
-            for ( const node of input ) {
-                if ( this.pselector.test(node) === this.target ) {
-                    output.push(node);
-                }
+        transpose(node, output) {
+            if ( this.pselector.test(node) === this.target ) {
+                output.push(node);
             }
-            return output;
         }
     };
     PSelectorIfTask.prototype.target = true;
 
     const PSelectorIfNotTask = class extends PSelectorIfTask {
-        constructor(task) {
-            super(task);
-            this.target = false;
-        }
     };
+    PSelectorIfNotTask.prototype.target = false;
 
     const PSelectorMatchesCSSTask = class {
         constructor(task) {
@@ -446,31 +508,31 @@ vAPI.DOMFilterer = (function() {
             }
             this.value = new RegExp(arg0, arg1);
         }
-        exec(input) {
-            const output = [];
-            for ( const node of input ) {
-                const style = window.getComputedStyle(node, this.pseudo);
-                if ( style === null ) { return null; } /* FF */
-                if ( this.value.test(style[this.name]) ) {
-                    output.push(node);
-                }
+        transpose(node, output) {
+            const style = window.getComputedStyle(node, this.pseudo);
+            if ( style !== null && this.value.test(style[this.name]) ) {
+                output.push(node);
             }
-            return output;
         }
     };
     PSelectorMatchesCSSTask.prototype.pseudo = null;
 
     const PSelectorMatchesCSSAfterTask = class extends PSelectorMatchesCSSTask {
-        constructor(task) {
-            super(task);
-            this.pseudo = ':after';
-        }
     };
+    PSelectorMatchesCSSAfterTask.prototype.pseudo = ':after';
 
     const PSelectorMatchesCSSBeforeTask = class extends PSelectorMatchesCSSTask {
+    };
+    PSelectorMatchesCSSBeforeTask.prototype.pseudo = ':before';
+
+    const PSelectorMinTextLengthTask = class {
         constructor(task) {
-            super(task);
-            this.pseudo = ':before';
+            this.min = task[1];
+        }
+        transpose(node, output) {
+            if ( node.textContent.length >= this.min ) {
+                output.push(node);
+            }
         }
     };
 
@@ -478,20 +540,15 @@ vAPI.DOMFilterer = (function() {
         constructor(task) {
             this.nth = task[1];
         }
-        exec(input) {
-            const output = [];
-            for ( let node of input ) {
-                let nth = this.nth;
-                for (;;) {
-                    node = node.parentElement;
-                    if ( node === null ) { break; }
-                    nth -= 1;
-                    if ( nth !== 0 ) { continue; }
-                    output.push(node);
-                    break;
-                }
+        transpose(node, output) {
+            let nth = this.nth;
+            for (;;) {
+                node = node.parentElement;
+                if ( node === null ) { return; }
+                nth -= 1;
+                if ( nth === 0 ) { break; }
             }
-            return output;
+            output.push(node);
         }
     };
 
@@ -499,25 +556,21 @@ vAPI.DOMFilterer = (function() {
         constructor(task) {
             this.spath = task[1];
         }
-        exec(input) {
-            const output = [];
-            for ( let node of input ) {
-                const parent = node.parentElement;
-                if ( parent === null ) { continue; }
-                let pos = 1;
-                for (;;) {
-                    node = node.previousElementSibling;
-                    if ( node === null ) { break; }
-                    pos += 1;
-                }
-                const nodes = parent.querySelectorAll(
-                    ':scope > :nth-child(' + pos + ')' + this.spath
-                );
-                for ( const node of nodes ) {
-                    output.push(node);
-                }
+        transpose(node, output) {
+            const parent = node.parentElement;
+            if ( parent === null ) { return; }
+            let pos = 1;
+            for (;;) {
+                node = node.previousElementSibling;
+                if ( node === null ) { break; }
+                pos += 1;
+            }
+            const nodes = parent.querySelectorAll(
+                `:scope > :nth-child(${pos})${this.spath}`
+            );
+            for ( const node of nodes ) {
+                output.push(node);
             }
-            return output;
         }
     };
 
@@ -542,17 +595,14 @@ vAPI.DOMFilterer = (function() {
                 filterer.onDOMChanged([ null ]);
             }
         }
-        exec(input) {
-            if ( input.length === 0 ) { return input; }
+        transpose(node, output) {
+            output.push(node);
+            if ( this.observed.has(node) ) { return; }
             if ( this.observer === null ) {
                 this.observer = new MutationObserver(this.handler);
             }
-            for ( const node of input ) {
-                if ( this.observed.has(node) ) { continue; }
-                this.observer.observe(node, this.observerOptions);
-                this.observed.add(node);
-            }
-            return input;
+            this.observer.observe(node, this.observerOptions);
+            this.observed.add(node);
         }
     };
 
@@ -561,23 +611,19 @@ vAPI.DOMFilterer = (function() {
             this.xpe = document.createExpression(task[1], null);
             this.xpr = null;
         }
-        exec(input) {
-            const output = [];
-            for ( const node of input ) {
-                this.xpr = this.xpe.evaluate(
-                    node,
-                    XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
-                    this.xpr
-                );
-                let j = this.xpr.snapshotLength;
-                while ( j-- ) {
-                    const node = this.xpr.snapshotItem(j);
-                    if ( node.nodeType === 1 ) {
-                        output.push(node);
-                    }
+        transpose(node, output) {
+            this.xpr = this.xpe.evaluate(
+                node,
+                XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
+                this.xpr
+            );
+            let j = this.xpr.snapshotLength;
+            while ( j-- ) {
+                const node = this.xpr.snapshotItem(j);
+                if ( node.nodeType === 1 ) {
+                    output.push(node);
                 }
             }
-            return output;
         }
     };
 
@@ -592,10 +638,11 @@ vAPI.DOMFilterer = (function() {
                     [ ':matches-css', PSelectorMatchesCSSTask ],
                     [ ':matches-css-after', PSelectorMatchesCSSAfterTask ],
                     [ ':matches-css-before', PSelectorMatchesCSSBeforeTask ],
+                    [ ':min-text-length', PSelectorMinTextLengthTask ],
                     [ ':not', PSelectorIfNotTask ],
                     [ ':nth-ancestor', PSelectorNthAncestorTask ],
                     [ ':spath', PSelectorSpathTask ],
-                    [ ':watch-attrs', PSelectorWatchAttrs ],
+                    [ ':watch-attr', PSelectorWatchAttrs ],
                     [ ':xpath', PSelectorXpathTask ],
                 ]);
             }
@@ -613,48 +660,51 @@ vAPI.DOMFilterer = (function() {
         }
         prime(input) {
             const root = input || document;
-            if ( this.selector !== '' ) {
-                return root.querySelectorAll(this.selector);
-            }
-            return [ root ];
+            if ( this.selector === '' ) { return [ root ]; }
+            return root.querySelectorAll(this.selector);
         }
         exec(input) {
             let nodes = this.prime(input);
             for ( const task of this.tasks ) {
                 if ( nodes.length === 0 ) { break; }
-                nodes = task.exec(nodes);
+                const transposed = [];
+                for ( const node of nodes ) {
+                    task.transpose(node, transposed);
+                }
+                nodes = transposed;
             }
             return nodes;
         }
         test(input) {
             const nodes = this.prime(input);
-            const AA = [ null ];
             for ( const node of nodes ) {
-                AA[0] = node;
-                let aa = AA;
+                let output = [ node ];
                 for ( const task of this.tasks ) {
-                    aa = task.exec(aa);
-                    if ( aa.length === 0 ) { break; }
+                    const transposed = [];
+                    for ( const node of output ) {
+                        task.transpose(node, transposed);
+                    }
+                    output = transposed;
+                    if ( output.length === 0 ) { break; }
                 }
-                if ( aa.length !== 0 ) { return true; }
+                if ( output.length !== 0 ) { return true; }
             }
             return false;
         }
     };
     PSelector.prototype.operatorToTaskMap = undefined;
 
-    const DOMProceduralFilterer = function(domFilterer) {
-        this.domFilterer = domFilterer;
-        this.domIsReady = false;
-        this.domIsWatched = false;
-        this.mustApplySelectors = false;
-        this.selectors = new Map();
-        this.hiddenNodes = new Set();
-    };
+    const DOMProceduralFilterer = class {
+        constructor(domFilterer) {
+            this.domFilterer = domFilterer;
+            this.domIsReady = false;
+            this.domIsWatched = false;
+            this.mustApplySelectors = false;
+            this.selectors = new Map();
+            this.hiddenNodes = new Set();
+        }
 
-    DOMProceduralFilterer.prototype = {
-
-        addProceduralSelectors: function(aa) {
+        addProceduralSelectors(aa) {
             const addedSelectors = [];
             let mustCommit = this.domIsWatched;
             for ( let i = 0, n = aa.length; i < n; i++ ) {
@@ -691,9 +741,9 @@ vAPI.DOMFilterer = (function() {
                     procedural: addedSelectors
                 });
             }
-        },
+        }
 
-        commitNow: function() {
+        commitNow() {
             if ( this.selectors.size === 0 || this.domIsReady === false ) {
                 return;
             }
@@ -738,18 +788,18 @@ vAPI.DOMFilterer = (function() {
                 this.domFilterer.unhideNode(node);
             }
             //console.timeEnd('procedural selectors/dom layout changed');
-        },
+        }
 
-        createProceduralFilter: function(o) {
+        createProceduralFilter(o) {
             return new PSelector(o);
-        },
+        }
 
-        onDOMCreated: function() {
+        onDOMCreated() {
             this.domIsReady = true;
             this.domFilterer.commitNow();
-        },
+        }
 
-        onDOMChanged: function(addedNodes, removedNodes) {
+        onDOMChanged(addedNodes, removedNodes) {
             if ( this.selectors.size === 0 ) { return; }
             this.mustApplySelectors =
                 this.mustApplySelectors ||
@@ -759,66 +809,60 @@ vAPI.DOMFilterer = (function() {
         }
     };
 
-    const DOMFiltererBase = vAPI.DOMFilterer;
+    const DOMFilterer = class extends vAPI.DOMFilterer {
+        constructor() {
+            super();
+            this.exceptions = [];
+            this.proceduralFilterer = new DOMProceduralFilterer(this);
+            this.hideNodeAttr = undefined;
+            this.hideNodeStyleSheetInjected = false;
+            if ( vAPI.domWatcher instanceof Object ) {
+                vAPI.domWatcher.addListener(this);
+            }
+        }
 
-    const domFilterer = function() {
-        DOMFiltererBase.call(this);
-        this.exceptions = [];
-        this.proceduralFilterer = new DOMProceduralFilterer(this);
-        this.hideNodeAttr = undefined;
-        this.hideNodeStyleSheetInjected = false;
+        commitNow() {
+            super.commitNow();
+            this.proceduralFilterer.commitNow();
+        }
 
-        // May or may not exist: cache locally since this may be called often.
-        this.baseOnDOMChanged = DOMFiltererBase.prototype.onDOMChanged;
+        addProceduralSelectors(aa) {
+            this.proceduralFilterer.addProceduralSelectors(aa);
+        }
 
-        if ( vAPI.domWatcher instanceof Object ) {
-            vAPI.domWatcher.addListener(this);
+        createProceduralFilter(o) {
+            return this.proceduralFilterer.createProceduralFilter(o);
+        }
+
+        getAllSelectors() {
+            const out = super.getAllSelectors();
+            out.procedural = Array.from(this.proceduralFilterer.selectors.values());
+            return out;
+        }
+
+        getAllExceptionSelectors() {
+            return this.exceptions.join(',\n');
+        }
+
+        onDOMCreated() {
+            if ( super.onDOMCreated instanceof Function ) {
+                super.onDOMCreated();
+            }
+            this.proceduralFilterer.onDOMCreated();
+        }
+
+        onDOMChanged() {
+            if ( super.onDOMChanged instanceof Function ) {
+                super.onDOMChanged.apply(this, arguments);
+            }
+            this.proceduralFilterer.onDOMChanged.apply(
+                this.proceduralFilterer,
+                arguments
+            );
         }
     };
-    domFilterer.prototype = Object.create(DOMFiltererBase.prototype);
-    domFilterer.prototype.constructor = domFilterer;
 
-    domFilterer.prototype.commitNow = function() {
-        DOMFiltererBase.prototype.commitNow.call(this);
-        this.proceduralFilterer.commitNow();
-    };
-
-    domFilterer.prototype.addProceduralSelectors = function(aa) {
-        this.proceduralFilterer.addProceduralSelectors(aa);
-    };
-
-    domFilterer.prototype.createProceduralFilter = function(o) {
-        return this.proceduralFilterer.createProceduralFilter(o);
-    };
-
-    domFilterer.prototype.getAllSelectors = function() {
-        const out = DOMFiltererBase.prototype.getAllSelectors.call(this);
-        out.procedural = Array.from(this.proceduralFilterer.selectors.values());
-        return out;
-    };
-
-    domFilterer.prototype.getAllExceptionSelectors = function() {
-        return this.exceptions.join(',\n');
-    };
-
-    domFilterer.prototype.onDOMCreated = function() {
-        if ( DOMFiltererBase.prototype.onDOMCreated !== undefined ) {
-            DOMFiltererBase.prototype.onDOMCreated.call(this);
-        }
-        this.proceduralFilterer.onDOMCreated();
-    };
-
-    domFilterer.prototype.onDOMChanged = function() {
-        if ( this.baseOnDOMChanged !== undefined ) {
-            this.baseOnDOMChanged.apply(this, arguments);
-        }
-        this.proceduralFilterer.onDOMChanged.apply(
-            this.proceduralFilterer,
-            arguments
-        );
-    };
-
-    return domFilterer;
+    return DOMFilterer;
 })();
 
 vAPI.domFilterer = new vAPI.DOMFilterer();
@@ -1274,6 +1318,10 @@ vAPI.domSurveyor = (function() {
                 );
                 mustCommit = true;
             }
+            selectors = result.excepted;
+            if ( Array.isArray(selectors) && selectors.length !== 0 ) {
+                domFilterer.exceptCSSRules(selectors);
+            }
         }
 
         if ( pendingNodes.stopped === false ) {
@@ -1458,6 +1506,7 @@ vAPI.bootstrap = (function() {
                 { injected: true }
             );
             domFilterer.addProceduralSelectors(cfeDetails.proceduralFilters);
+            domFilterer.exceptCSSRules(cfeDetails.exceptedFilters);
         }
 
         if ( cfeDetails.networkFilters.length !== 0 ) {
