@@ -1,7 +1,7 @@
 /*******************************************************************************
 
-    uBlock - a browser extension to block requests.
-    Copyright (C) 2015-2017 Raymond Hill
+    uBlock Origin - a browser extension to block requests.
+    Copyright (C) 2015-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,34 +26,20 @@
 
 µBlock.logger = (function() {
 
-    var LogEntry = function(args) {
-        this.init(args);
-    };
-
-    LogEntry.prototype.init = function(args) {
-        this.tstamp = Date.now();
-        this.tab = args[0] || '';
-        this.cat = args[1] || '';
-        this.d0 = args[2];
-        this.d1 = args[3];
-        this.d2 = args[4];
-        this.d3 = args[5];
-        this.d4 = args[6];
-    };
-
-    var buffer = null;
-    var lastReadTime = 0;
-    var writePtr = 0;
+    let buffer = null;
+    let lastReadTime = 0;
+    let writePtr = 0;
 
     // After 60 seconds without being read, a buffer will be considered
     // unused, and thus removed from memory.
-    var logBufferObsoleteAfter = 30 * 1000;
+    const logBufferObsoleteAfter = 30 * 1000;
 
-    var janitor = function() {
+    const janitor = ( ) => {
         if (
             buffer !== null &&
             lastReadTime < (Date.now() - logBufferObsoleteAfter)
         ) {
+            api.enabled = false;
             buffer = null;
             writePtr = 0;
             api.ownerId = undefined;
@@ -64,31 +50,37 @@
         }
     };
 
-    var api = {
+    const boxEntry = function(details) {
+        if ( details.tstamp === undefined ) {
+            details.tstamp = Date.now();
+        }
+        return JSON.stringify(details);
+    };
+
+    const api = {
+        enabled: false,
         ownerId: undefined,
-        writeOne: function() {
+        writeOne: function(details) {
             if ( buffer === null ) { return; }
             if ( writePtr === buffer.length ) {
-                buffer.push(new LogEntry(arguments));
+                buffer.push(boxEntry(details));
             } else {
-                buffer[writePtr].init(arguments);
+                buffer[writePtr] = boxEntry(details);
             }
             writePtr += 1;
         },
         readAll: function(ownerId) {
             this.ownerId = ownerId;
             if ( buffer === null ) {
+                this.enabled = true;
                 buffer = [];
                 vAPI.setTimeout(janitor, logBufferObsoleteAfter);
             }
-            var out = buffer.slice(0, writePtr);
+            const out = buffer.slice(0, writePtr);
             writePtr = 0;
             lastReadTime = Date.now();
             return out;
         },
-        isEnabled: function() {
-            return buffer !== null;
-        }
     };
 
     return api;
