@@ -24,40 +24,43 @@
 
 'use strict';
 
+// https://github.com/uBlockOrigin/uBlock-issues/issues/710
+//   Listeners have a name and a "privileged" status.
+//   The nameless default handler is always deemed "privileged".
+//   Messages from privileged ports must never relayed to listeners
+//   which are not privileged.
+
+/******************************************************************************/
+/******************************************************************************/
+
 // Default handler
+//      priviledged
 
-(function() {
+{
+// >>>>> start of local scope
 
-/******************************************************************************/
+const µb = µBlock;
 
-var µb = µBlock;
-
-/******************************************************************************/
-
-var getDomainNames = function(targets) {
-    var out = [];
-    var µburi = µb.URI;
-    var target, domain;
-    for ( var i = 0; i < targets.length; i++ ) {
-        target = targets[i];
-        if ( target.indexOf('/') !== -1 ) {
-            domain = µburi.domainFromURI(target) || '';
-        } else {
-            domain = µburi.domainFromHostname(target) || target;
-        }
-        out.push(domain);
-    }
-    return out;
+const getDomainNames = function(targets) {
+    const µburi = µb.URI;
+    return targets.map(target => {
+        if ( typeof target !== 'string' ) { return ''; }
+        return target.indexOf('/') !== -1
+            ? µburi.domainFromURI(target) || ''
+            : µburi.domainFromHostname(target) || target;
+    });
 };
 
-/******************************************************************************/
-
 const onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
     case 'getAssetContent':
         // https://github.com/chrisaljoudi/uBlock/issues/417
-        µb.assets.get(request.url, { dontCache: true }, callback);
+        µb.assets.get(
+            request.url,
+            { dontCache: true, needSourceURL: true },
+            callback
+        );
         return;
 
     case 'listsFromNetFilter':
@@ -84,8 +87,6 @@ const onMessage = function(request, sender, callback) {
         break;
     }
 
-    const tabId = sender && sender.tab ? sender.tab.id : 0;
-
     // Sync
     var response;
 
@@ -94,18 +95,8 @@ const onMessage = function(request, sender, callback) {
         response = µb.applyFilterListSelection(request);
         break;
 
-    case 'compileCosmeticFilterSelector':
-        response = µb.staticExtFilteringEngine.compileSelector(request.selector);
-        break;
-
-    case 'cosmeticFiltersInjected':
-        µb.cosmeticFilteringEngine.addToSelectorCache(request);
-        break;
-
     case 'createUserFilter':
-        µb.appendUserFilters(request.filters, request);
-        // https://github.com/gorhill/uBlock/issues/1786
-        µb.cosmeticFilteringEngine.removeFromSelectorCache(request.pageDomain);
+        µb.createUserFilters(request);
         break;
 
     case 'forceUpdateAssets':
@@ -128,7 +119,8 @@ const onMessage = function(request, sender, callback) {
 
     case 'getWhitelist':
         response = {
-            whitelist: µb.stringFromWhitelist(µb.netWhitelist),
+            whitelist: µb.arrayFromWhitelist(µb.netWhitelist),
+            whitelistDefault: µb.netWhitelistDefault,
             reBadHostname: µb.reWhitelistBadHostname.source,
             reHostnameExtractor: µb.reWhitelistHostnameExtractor.source
         };
@@ -144,13 +136,6 @@ const onMessage = function(request, sender, callback) {
         µb.openNewTab(request.details);
         break;
 
-    case 'mouseClick':
-        µb.mouseEventRegister.tabId = tabId;
-        µb.mouseEventRegister.x = request.x;
-        µb.mouseEventRegister.y = request.y;
-        µb.mouseEventRegister.url = request.url;
-        break;
-
     case 'reloadTab':
         if ( vAPI.isBehindTheSceneTabId(request.tabId) === false ) {
             vAPI.tabs.reload(request.tabId, request.bypassCache === true);
@@ -160,10 +145,6 @@ const onMessage = function(request, sender, callback) {
         }
         break;
 
-    case 'scriptletResponse':
-        µb.scriptlets.report(tabId, request.scriptlet, request.response);
-        break;
-
     case 'setWhitelist':
         µb.netWhitelist = µb.whitelistFromString(request.whitelist);
         µb.saveWhitelist();
@@ -186,51 +167,43 @@ const onMessage = function(request, sender, callback) {
 
 vAPI.messaging.setup(onMessage);
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 /******************************************************************************/
 /******************************************************************************/
 
-// channel: popupPanel
+// Channel:
+//      popupPanel
+//      privileged
 
-(function() {
+{
+// >>>>> start of local scope
 
-/******************************************************************************/
+const µb = µBlock;
 
-var µb = µBlock;
-
-/******************************************************************************/
-
-var getHostnameDict = function(hostnameToCountMap) {
-    var r = Object.create(null),
-        domainEntry,
-        domainFromHostname = µb.URI.domainFromHostname,
-        domain, blockCount, allowCount,
-        hostname, counts;
+const getHostnameDict = function(hostnameToCountMap) {
+    const r = Object.create(null);
+    const domainFromHostname = µb.URI.domainFromHostname;
     // Note: destructuring assignment not supported before Chromium 49.
-    for ( var entry of hostnameToCountMap ) {
-        hostname = entry[0];
+    for ( const [ hostname, hnCounts ] of hostnameToCountMap ) {
         if ( r[hostname] !== undefined ) { continue; }
-        domain = domainFromHostname(hostname) || hostname;
-        counts = hostnameToCountMap.get(domain) || 0;
-        blockCount = counts & 0xFFFF;
-        allowCount = counts >>> 16 & 0xFFFF;
+        const domain = domainFromHostname(hostname) || hostname;
+        const dnCounts = hostnameToCountMap.get(domain) || 0;
+        let blockCount = dnCounts & 0xFFFF;
+        let allowCount = dnCounts >>> 16 & 0xFFFF;
         if ( r[domain] === undefined ) {
-            domainEntry = r[domain] = {
+            r[domain] = {
                 domain: domain,
                 blockCount: blockCount,
                 allowCount: allowCount,
                 totalBlockCount: blockCount,
                 totalAllowCount: allowCount
             };
-        } else {
-            domainEntry = r[domain];
         }
-        counts = entry[1];
-        blockCount = counts & 0xFFFF;
-        allowCount = counts >>> 16 & 0xFFFF;
+        const domainEntry = r[domain];
+        blockCount = hnCounts & 0xFFFF;
+        allowCount = hnCounts >>> 16 & 0xFFFF;
         domainEntry.totalBlockCount += blockCount;
         domainEntry.totalAllowCount += allowCount;
         if ( hostname === domain ) { continue; }
@@ -245,9 +218,7 @@ var getHostnameDict = function(hostnameToCountMap) {
     return r;
 };
 
-/******************************************************************************/
-
-var getFirewallRules = function(srcHostname, desHostnames) {
+const getFirewallRules = function(srcHostname, desHostnames) {
     var r = {};
     var df = µb.sessionFirewall;
     r['/ * *'] = df.lookupRuleData('*', '*', '*');
@@ -257,32 +228,38 @@ var getFirewallRules = function(srcHostname, desHostnames) {
     r['/ * 1p-script'] = df.lookupRuleData('*', '*', '1p-script');
     r['/ * 3p-script'] = df.lookupRuleData('*', '*', '3p-script');
     r['/ * 3p-frame'] = df.lookupRuleData('*', '*', '3p-frame');
-    if ( typeof srcHostname !== 'string' ) {
-        return r;
-    }
+    if ( typeof srcHostname !== 'string' ) { return r; }
 
     r['. * *'] = df.lookupRuleData(srcHostname, '*', '*');
     r['. * image'] = df.lookupRuleData(srcHostname, '*', 'image');
     r['. * 3p'] = df.lookupRuleData(srcHostname, '*', '3p');
-    r['. * inline-script'] = df.lookupRuleData(srcHostname, '*', 'inline-script');
+    r['. * inline-script'] = df.lookupRuleData(srcHostname,
+        '*',
+        'inline-script'
+    );
     r['. * 1p-script'] = df.lookupRuleData(srcHostname, '*', '1p-script');
     r['. * 3p-script'] = df.lookupRuleData(srcHostname, '*', '3p-script');
     r['. * 3p-frame'] = df.lookupRuleData(srcHostname, '*', '3p-frame');
 
-    for ( var desHostname in desHostnames ) {
-        r['/ ' + desHostname + ' *'] = df.lookupRuleData('*', desHostname, '*');
-        r['. ' + desHostname + ' *'] = df.lookupRuleData(srcHostname, desHostname, '*');
+    for ( const desHostname in desHostnames ) {
+        r[`/ ${desHostname} *`] = df.lookupRuleData(
+            '*',
+            desHostname,
+            '*'
+        );
+        r[`. ${desHostname} *`] = df.lookupRuleData(
+            srcHostname,
+            desHostname,
+            '*'
+        );
     }
     return r;
 };
 
-/******************************************************************************/
-
-var popupDataFromTabId = function(tabId, tabTitle) {
-    let tabContext = µb.tabContextManager.mustLookup(tabId),
-        rootHostname = tabContext.rootHostname;
-
-    let r = {
+const popupDataFromTabId = function(tabId, tabTitle) {
+    const tabContext = µb.tabContextManager.mustLookup(tabId);
+    const rootHostname = tabContext.rootHostname;
+    const r = {
         advancedUserEnabled: µb.userSettings.advancedUserEnabled,
         appName: vAPI.app.name,
         appVersion: vAPI.app.version,
@@ -306,13 +283,13 @@ var popupDataFromTabId = function(tabId, tabTitle) {
         tooltipsDisabled: µb.userSettings.tooltipsDisabled
     };
 
-    let pageStore = µb.pageStoreFromTabId(tabId);
+    const pageStore = µb.pageStoreFromTabId(tabId);
     if ( pageStore ) {
         // https://github.com/gorhill/uBlock/issues/2105
-        //   Be sure to always include the current page's hostname -- it might
-        //   not be present when the page itself is pulled from the browser's
-        //   short-term memory cache. This needs to be done before calling
-        //   getHostnameDict().
+        //   Be sure to always include the current page's hostname -- it
+        //   might not be present when the page itself is pulled from the
+        //   browser's short-term memory cache. This needs to be done
+        //   before calling getHostnameDict().
         if (
             pageStore.hostnameToCountMap.has(rootHostname) === false &&
             µb.URI.isNetworkURI(tabContext.rawURL)
@@ -326,14 +303,29 @@ var popupDataFromTabId = function(tabId, tabTitle) {
         r.contentLastModified = pageStore.contentLastModified;
         r.firewallRules = getFirewallRules(rootHostname, r.hostnameDict);
         r.canElementPicker = µb.URI.isNetworkURI(r.rawURL);
-        r.noPopups = µb.sessionSwitches.evaluateZ('no-popups', rootHostname);
+        r.noPopups = µb.sessionSwitches.evaluateZ(
+            'no-popups',
+            rootHostname
+        );
         r.popupBlockedCount = pageStore.popupBlockedCount;
-        r.noCosmeticFiltering = µb.sessionSwitches.evaluateZ('no-cosmetic-filtering', rootHostname);
-        r.noLargeMedia = µb.sessionSwitches.evaluateZ('no-large-media', rootHostname);
+        r.noCosmeticFiltering = µb.sessionSwitches.evaluateZ(
+            'no-cosmetic-filtering',
+            rootHostname
+        );
+        r.noLargeMedia = µb.sessionSwitches.evaluateZ(
+            'no-large-media',
+            rootHostname
+        );
         r.largeMediaCount = pageStore.largeMediaCount;
-        r.noRemoteFonts = µb.sessionSwitches.evaluateZ('no-remote-fonts', rootHostname);
+        r.noRemoteFonts = µb.sessionSwitches.evaluateZ(
+            'no-remote-fonts',
+            rootHostname
+        );
         r.remoteFontCount = pageStore.remoteFontCount;
-        r.noScripting = µb.sessionSwitches.evaluateZ('no-scripting', rootHostname);
+        r.noScripting = µb.sessionSwitches.evaluateZ(
+            'no-scripting',
+            rootHostname
+        );
     } else {
         r.hostnameDict = {};
         r.firewallRules = getFirewallRules();
@@ -353,9 +345,7 @@ var popupDataFromTabId = function(tabId, tabTitle) {
     return r;
 };
 
-/******************************************************************************/
-
-var popupDataFromRequest = function(request, callback) {
+const popupDataFromRequest = function(request, callback) {
     if ( request.tabId ) {
         callback(popupDataFromTabId(request.tabId, ''));
         return;
@@ -373,10 +363,8 @@ var popupDataFromRequest = function(request, callback) {
     });
 };
 
-/******************************************************************************/
-
-var onMessage = function(request, sender, callback) {
-    var pageStore;
+const onMessage = function(request, sender, callback) {
+    let pageStore;
 
     // Async
     switch ( request.what ) {
@@ -389,7 +377,7 @@ var onMessage = function(request, sender, callback) {
     }
 
     // Sync
-    var response;
+    let response;
 
     switch ( request.what ) {
     case 'getPopupLazyData':
@@ -426,6 +414,7 @@ var onMessage = function(request, sender, callback) {
             request.srcHostname,
             'net'
         );
+        µb.updateToolbarIcon(request.tabId, 0b100);
         response = popupDataFromTabId(request.tabId);
         break;
 
@@ -462,8 +451,12 @@ var onMessage = function(request, sender, callback) {
     case 'toggleNetFiltering':
         pageStore = µb.pageStoreFromTabId(request.tabId);
         if ( pageStore ) {
-            pageStore.toggleNetFilteringSwitch(request.url, request.scope, request.state);
-            µb.updateToolbarIcon(request.tabId, 0x03);
+            pageStore.toggleNetFilteringSwitch(
+                request.url,
+                request.scope,
+                request.state
+            );
+            µb.updateToolbarIcon(request.tabId, 0b111);
         }
         break;
 
@@ -474,22 +467,28 @@ var onMessage = function(request, sender, callback) {
     callback(response);
 };
 
-vAPI.messaging.listen('popupPanel', onMessage);
+vAPI.messaging.listen({
+    name: 'popupPanel',
+    listener: onMessage,
+    privileged: true,
+});
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 /******************************************************************************/
 /******************************************************************************/
 
-// channel: contentscript
+// Channel:
+//      contentscript
+//      unprivileged
 
-(function() {
+{
+// >>>>> start of local scope
 
-/******************************************************************************/
+const µb = µBlock;
 
-var onMessage = function(request, sender, callback) {
+const onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
     default:
@@ -497,7 +496,6 @@ var onMessage = function(request, sender, callback) {
     }
 
     // Sync
-    const µb = µBlock;
     let response,
         tabId, frameId,
         pageStore = null;
@@ -509,6 +507,10 @@ var onMessage = function(request, sender, callback) {
     }
 
     switch ( request.what ) {
+    case 'cosmeticFiltersInjected':
+        µb.cosmeticFilteringEngine.addToSelectorCache(request);
+        break;
+
     case 'getCollapsibleBlockedRequests':
         response = {
             id: request.id,
@@ -525,6 +527,13 @@ var onMessage = function(request, sender, callback) {
         }
         break;
 
+    case 'mouseClick':
+        µb.mouseEventRegister.tabId = tabId;
+        µb.mouseEventRegister.x = request.x;
+        µb.mouseEventRegister.y = request.y;
+        µb.mouseEventRegister.url = request.url;
+        break;
+
     case 'shouldRenderNoscriptTags':
         if ( pageStore === null ) { break; }
         const fctxt = µb.filteringContext.fromTabId(tabId);
@@ -548,12 +557,29 @@ var onMessage = function(request, sender, callback) {
         ) {
             break;
         }
+        const noCosmeticFiltering = pageStore.noCosmeticFiltering === true;
         response = {
             collapseBlocked: µb.userSettings.collapseBlocked,
-            noCosmeticFiltering: pageStore.noCosmeticFiltering === true,
-            noGenericCosmeticFiltering:
-                pageStore.noGenericCosmeticFiltering === true
+            noCosmeticFiltering,
+            noGenericCosmeticFiltering: noCosmeticFiltering,
         };
+        // https://github.com/uBlockOrigin/uAssets/issues/5704
+        //   `generichide` must be evaluated in the frame context.
+        if ( noCosmeticFiltering === false ) {
+            const genericHide =
+                µb.staticNetFilteringEngine.matchStringGenericHide(request.url);
+            response.noGenericCosmeticFiltering = genericHide === 2;
+            if ( genericHide !== 0 && µb.logger.enabled ) {
+                µBlock.filteringContext
+                    .duplicate()
+                    .fromTabId(tabId)
+                    .setURL(request.url)
+                    .setRealm('network')
+                    .setType('generichide')
+                    .setFilter(µb.staticNetFilteringEngine.toLogData())
+                    .toLogger();
+            }
+        }
         request.tabId = tabId;
         request.frameId = frameId;
         request.hostname = µb.URI.hostnameFromURI(request.url);
@@ -567,23 +593,17 @@ var onMessage = function(request, sender, callback) {
         if ( µb.canInjectScriptletsNow === false ) {
             response.scriptlets = µb.scriptletFilteringEngine.retrieve(request);
         }
-        if ( µb.logger.enabled ) {
-            if ( response.noCosmeticFiltering !== true ) {
-                µb.logCosmeticFilters(tabId, frameId);
-            }
-            µb.logInlineScript(tabId, frameId);
+        if ( µb.logger.enabled && response.noCosmeticFiltering !== true ) {
+            µb.logCosmeticFilters(tabId, frameId);
         }
         break;
 
     case 'retrieveGenericCosmeticSelectors':
-        if ( pageStore && pageStore.getGenericCosmeticFilteringSwitch() ) {
-            request.tabId = tabId;
-            request.frameId = frameId;
-            response = {
-                result: µb.cosmeticFilteringEngine
-                          .retrieveGenericSelectors(request)
-            };
-        }
+        request.tabId = tabId;
+        request.frameId = frameId;
+        response = {
+            result: µb.cosmeticFilteringEngine.retrieveGenericSelectors(request)
+        };
         break;
 
     default:
@@ -593,20 +613,23 @@ var onMessage = function(request, sender, callback) {
     callback(response);
 };
 
-vAPI.messaging.listen('contentscript', onMessage);
+vAPI.messaging.listen({
+    name: 'contentscript',
+    listener: onMessage,
+});
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 /******************************************************************************/
 /******************************************************************************/
 
-// channel: elementPicker
+// Channel:
+//      elementPicker
+//      unprivileged
 
-(function() {
-
-/******************************************************************************/
+{
+// >>>>> start of local scope
 
 const onMessage = function(request, sender, callback) {
     const µb = µBlock;
@@ -658,6 +681,18 @@ const onMessage = function(request, sender, callback) {
     let response;
 
     switch ( request.what ) {
+    case 'compileCosmeticFilterSelector':
+        response = µb.staticExtFilteringEngine.compileSelector(
+            request.selector
+        );
+        break;
+
+    // https://github.com/gorhill/uBlock/issues/3497
+    //   This needs to be removed once issue is fixed.
+    case 'createUserFilter':
+        µb.createUserFilters(request);
+        break;
+
     case 'elementPickerEprom':
         µb.epickerEprom = request;
         break;
@@ -669,22 +704,25 @@ const onMessage = function(request, sender, callback) {
     callback(response);
 };
 
-vAPI.messaging.listen('elementPicker', onMessage);
+vAPI.messaging.listen({
+    name: 'elementPicker',
+    listener: onMessage,
+});
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 /******************************************************************************/
 /******************************************************************************/
 
-// channel: cloudWidget
+// Channel:
+//      cloudWidget
+//      privileged
 
-(function() {
+{
+// >>>>> start of local scope
 
-/******************************************************************************/
-
-var onMessage = function(request, sender, callback) {
+const onMessage = function(request, sender, callback) {
     // Cloud storage support is optional.
     if ( µBlock.cloudStorageSupported !== true ) {
         callback();
@@ -715,7 +753,7 @@ var onMessage = function(request, sender, callback) {
     }
 
     // Sync
-    var response;
+    let response;
 
     switch ( request.what ) {
     // For when cloud storage is disabled.
@@ -731,30 +769,31 @@ var onMessage = function(request, sender, callback) {
     callback(response);
 };
 
-vAPI.messaging.listen('cloudWidget', onMessage);
+vAPI.messaging.listen({
+    name: 'cloudWidget',
+    listener: onMessage,
+    privileged: true,
+});
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 /******************************************************************************/
 /******************************************************************************/
 
-// channel: dashboard
+// Channel:
+//      dashboard
+//      privileged
 
-(function() {
-
-/******************************************************************************/
+{
+// >>>>> start of local scope
 
 const µb = µBlock;
 
-/******************************************************************************/
-
 // Settings
-
-var getLocalData = function(callback) {
-    var onStorageInfoReady = function(bytesInUse) {
-        var o = µb.restoreBackupSettings;
+const getLocalData = function(callback) {
+    const onStorageInfoReady = function(bytesInUse) {
+        const o = µb.restoreBackupSettings;
         callback({
             storageUsed: bytesInUse,
             lastRestoreFile: o.lastRestoreFile,
@@ -769,13 +808,15 @@ var getLocalData = function(callback) {
     µb.getBytesInUse(onStorageInfoReady);
 };
 
-var backupUserData = function(callback) {
-    var userData = {
+const backupUserData = function(callback) {
+    const userData = {
         timeStamp: Date.now(),
         version: vAPI.app.version,
         userSettings: µb.userSettings,
         selectedFilterLists: µb.selectedFilterLists,
         hiddenSettings: µb.hiddenSettings,
+        whitelist: µb.arrayFromWhitelist(µb.netWhitelist),
+        // String representation eventually to be deprecated
         netWhitelist: µb.stringFromWhitelist(µb.netWhitelist),
         dynamicFilteringString: µb.permanentFirewall.toString(),
         urlFilteringString: µb.permanentURLFiltering.toString(),
@@ -783,9 +824,9 @@ var backupUserData = function(callback) {
         userFilters: ''
     };
 
-    var onUserFiltersReady = function(details) {
+    const onUserFiltersReady = function(details) {
         userData.userFilters = details.content;
-        var filename = vAPI.i18n('aboutBackupFilename')
+        const filename = vAPI.i18n('aboutBackupFilename')
             .replace('{{datetime}}', µb.dateNowToSensibleString())
             .replace(/ +/g, '_');
         µb.restoreBackupSettings.lastBackupFile = filename;
@@ -819,9 +860,19 @@ const restoreUserData = function(request) {
                 userData.hiddenSettingsString || ''
             );
         }
+        // Whitelist directives can be represented as an array or as a
+        // (eventually to be deprecated) string.
+        let whitelist = userData.whitelist;
+        if (
+            Array.isArray(whitelist) === false &&
+            typeof userData.netWhitelist === 'string' &&
+            userData.netWhitelist !== ''
+        ) {
+            whitelist = userData.netWhitelist.split('\n');
+        }
         vAPI.storage.set({
             hiddenSettings: hiddenSettings,
-            netWhitelist: userData.netWhitelist || '',
+            netWhitelist: whitelist || [],
             dynamicFilteringString: userData.dynamicFilteringString || '',
             urlFilteringString: userData.urlFilteringString || '',
             hostnameSwitchesString: userData.hostnameSwitchesString || '',
@@ -851,7 +902,6 @@ const restoreUserData = function(request) {
 
 // Remove all stored data but keep global counts, people can become
 // quite attached to numbers
-
 const resetUserData = function() {
     let count = 3;
     const countdown = ( ) => {
@@ -866,28 +916,24 @@ const resetUserData = function() {
     vAPI.localStorage.removeItem('immediateHiddenSettings');
 };
 
-/******************************************************************************/
-
 // 3rd-party filters
-
-var prepListEntries = function(entries) {
-    var µburi = µb.URI;
-    var entry, hn;
-    for ( var k in entries ) {
+const prepListEntries = function(entries) {
+    const µburi = µb.URI;
+    for ( const k in entries ) {
         if ( entries.hasOwnProperty(k) === false ) { continue; }
-        entry = entries[k];
+        const entry = entries[k];
         if ( typeof entry.supportURL === 'string' && entry.supportURL !== '' ) {
             entry.supportName = µburi.hostnameFromURI(entry.supportURL);
         } else if ( typeof entry.homeURL === 'string' && entry.homeURL !== '' ) {
-            hn = µburi.hostnameFromURI(entry.homeURL);
-            entry.supportURL = 'http://' + hn + '/';
+            const hn = µburi.hostnameFromURI(entry.homeURL);
+            entry.supportURL = `http://${hn}/`;
             entry.supportName = µburi.domainFromHostname(hn);
         }
     }
 };
 
-var getLists = function(callback) {
-    var r = {
+const getLists = function(callback) {
+    const r = {
         autoUpdate: µb.userSettings.autoUpdate,
         available: null,
         cache: null,
@@ -895,16 +941,17 @@ var getLists = function(callback) {
         current: µb.availableFilterLists,
         externalLists: µb.userSettings.externalLists,
         ignoreGenericCosmeticFilters: µb.userSettings.ignoreGenericCosmeticFilters,
+        isUpdating: µb.assets.isUpdating(),
         netFilterCount: µb.staticNetFilteringEngine.getFilterCount(),
         parseCosmeticFilters: µb.userSettings.parseAllABPHideFilters,
         userFiltersPath: µb.userFiltersPath
     };
-    var onMetadataReady = function(entries) {
+    const onMetadataReady = function(entries) {
         r.cache = entries;
         prepListEntries(r.cache);
         callback(r);
     };
-    var onLists = function(lists) {
+    const onLists = function(lists) {
         r.available = lists;
         prepListEntries(r.available);
         µb.assets.metadata(onMetadataReady);
@@ -912,11 +959,8 @@ var getLists = function(callback) {
     µb.getAvailableLists(onLists);
 };
 
-/******************************************************************************/
-
 // My rules
-
-var getRules = function() {
+const getRules = function() {
     return {
         permanentRules:
             µb.permanentFirewall.toArray().concat(
@@ -931,7 +975,7 @@ var getRules = function() {
     };
 };
 
-var modifyRuleset = function(details) {
+const modifyRuleset = function(details) {
     let swRuleset, hnRuleset, urlRuleset;
     if ( details.permanent ) {
         swRuleset = µb.permanentSwitches;
@@ -978,11 +1022,8 @@ var modifyRuleset = function(details) {
     }
 };
 
-/******************************************************************************/
-
 // Shortcuts pane
-
-let getShortcuts = function(callback) {
+const getShortcuts = function(callback) {
     if ( µb.canUseShortcuts === false ) {
         return callback([]);
     }
@@ -1003,7 +1044,7 @@ let getShortcuts = function(callback) {
     });
 };
 
-let setShortcut = function(details) {
+const setShortcut = function(details) {
     if  ( µb.canUpdateShortcuts === false ) { return; }
     if ( details.shortcut === undefined ) {
         vAPI.commands.reset(details.name);
@@ -1015,9 +1056,7 @@ let setShortcut = function(details) {
     vAPI.storage.set({ commandShortcuts: Array.from(µb.commandShortcuts) });
 };
 
-/******************************************************************************/
-
-var onMessage = function(request, sender, callback) {
+const onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
     case 'backupUserData':
@@ -1072,11 +1111,6 @@ var onMessage = function(request, sender, callback) {
     case 'purgeCache':
         µb.assets.purge(request.assetKey);
         µb.assets.remove('compiled/' + request.assetKey);
-        // https://github.com/gorhill/uBlock/pull/2314#issuecomment-278716960
-        if ( request.assetKey === 'ublock-filters' ) {
-            µb.assets.purge('ublock-resources');
-            µb.redirectEngine.invalidateResourcesSelfie();
-        }
         break;
 
     case 'readHiddenSettings':
@@ -1106,26 +1140,28 @@ var onMessage = function(request, sender, callback) {
     callback(response);
 };
 
-vAPI.messaging.listen('dashboard', onMessage);
+vAPI.messaging.listen({
+    name: 'dashboard',
+    listener: onMessage,
+    privileged: true,
+});
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 /******************************************************************************/
 /******************************************************************************/
 
-// channel: loggerUI
+// Channel:
+//      loggerUI
+//      privileged
 
-(function() {
-
-/******************************************************************************/
+{
+// >>>>> start of local scope
 
 const µb = µBlock;
 const extensionOriginURL = vAPI.getURL('');
 
-/******************************************************************************/
-
 const getLoggerData = function(details, activeTabId, callback) {
     const response = {
         colorBlind: µb.userSettings.colorBlindFriendly,
@@ -1177,8 +1213,6 @@ const getLoggerData = function(details, activeTabId, callback) {
     callback(response);
 };
 
-/******************************************************************************/
-
 const getURLFilteringData = function(details) {
     const colors = {};
     const response = {
@@ -1211,8 +1245,6 @@ const getURLFilteringData = function(details) {
     return response;
 };
 
-/******************************************************************************/
-
 const onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
@@ -1270,22 +1302,28 @@ const onMessage = function(request, sender, callback) {
     callback(response);
 };
 
-vAPI.messaging.listen('loggerUI', onMessage);
+vAPI.messaging.listen({
+    name: 'loggerUI',
+    listener: onMessage,
+    privileged: true,
+});
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 /******************************************************************************/
 /******************************************************************************/
 
-// channel: documentBlocked
+// Channel:
+//      documentBlocked
+//      privileged
 
-(function() {
+{
+// >>>>> start of local scope
 
-/******************************************************************************/
+const onMessage = function(request, sender, callback) {
+    const tabId = sender && sender.tab ? sender.tab.id : 0;
 
-var onMessage = function(request, sender, callback) {
     // Async
     switch ( request.what ) {
     default:
@@ -1296,6 +1334,10 @@ var onMessage = function(request, sender, callback) {
     let response;
 
     switch ( request.what ) {
+    case 'closeThisTab':
+        vAPI.tabs.remove(tabId);
+        break;
+
     case 'temporarilyWhitelistDocument':
         µBlock.webRequest.strictBlockBypass(request.hostname);
         break;
@@ -1307,26 +1349,28 @@ var onMessage = function(request, sender, callback) {
     callback(response);
 };
 
-vAPI.messaging.listen('documentBlocked', onMessage);
+vAPI.messaging.listen({
+    name: 'documentBlocked',
+    listener: onMessage,
+    privileged: true,
+});
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 /******************************************************************************/
 /******************************************************************************/
 
-// channel: scriptlets
+// Channel:
+//      scriptlets
+//      unprivileged
 
-(function() {
-
-/******************************************************************************/
+{
+// >>>>> start of local scope
 
 const µb = µBlock;
 const broadcastTimers = new Map();
 
-/******************************************************************************/
-
 const domSurveyFinalReport = function(tabId) {
     broadcastTimers.delete(tabId + '-domSurveyReport');
 
@@ -1341,8 +1385,6 @@ const domSurveyFinalReport = function(tabId) {
     });
 };
 
-/******************************************************************************/
-
 const logCosmeticFilters = function(tabId, details) {
     if ( µb.logger.enabled === false ) { return; }
 
@@ -1355,14 +1397,101 @@ const logCosmeticFilters = function(tabId, details) {
          .setDocOriginFromURL(details.frameURL)
          .setFilter(filter);
     for ( const selector of details.matchedSelectors.sort() ) {
-        filter.raw = '##' + selector;
+        filter.raw = selector;
         fctxt.toLogger();
     }
 };
 
-/******************************************************************************/
+const logCSPViolations = function(pageStore, request) {
+    if ( µb.logger.enabled === false || pageStore === null ) {
+        return false;
+    }
+    if ( request.violations.length === 0 ) {
+        return true;
+    }
 
-var onMessage = function(request, sender, callback) {
+    const fctxt = µb.filteringContext.duplicate();
+    fctxt.fromTabId(pageStore.tabId)
+         .setRealm('network')
+         .setDocOriginFromURL(request.docURL)
+         .setURL(request.docURL);
+
+    let cspData = pageStore.extraData.get('cspData');
+    if ( cspData === undefined ) {
+        cspData = new Map();
+
+        const policies = [];
+        const logData = [];
+        µb.staticNetFilteringEngine.matchAndFetchData(
+            'csp',
+            request.docURL,
+            policies,
+            logData
+        );
+        for ( let i = 0; i < policies.length; i++ ) {
+            cspData.set(policies[i], logData[i]);
+        }
+        
+        fctxt.type = 'inline-script';
+        fctxt.filter = undefined;
+        if ( pageStore.filterRequest(fctxt) === 1 ) {
+            cspData.set(µb.cspNoInlineScript, fctxt.filter);
+        }
+
+        fctxt.type = 'script';
+        fctxt.filter = undefined;
+        if ( pageStore.filterScripting(fctxt, true) === 1 ) {
+            cspData.set(µb.cspNoScripting, fctxt.filter);
+        }
+    
+        fctxt.type = 'inline-font';
+        fctxt.filter = undefined;
+        if ( pageStore.filterRequest(fctxt) === 1 ) {
+            cspData.set(µb.cspNoInlineFont, fctxt.filter);
+        }
+
+        if ( cspData.size === 0 ) { return false; }
+
+        pageStore.extraData.set('cspData', cspData);
+    }
+
+    const typeMap = logCSPViolations.policyDirectiveToTypeMap;
+    for ( const json of request.violations ) {
+        const violation = JSON.parse(json);
+        let type = typeMap.get(violation.directive);
+        if ( type === undefined ) { continue; }
+        const logData = cspData.get(violation.policy);
+        if ( logData === undefined ) { continue; }
+        if ( /^[\w.+-]+:\/\//.test(violation.url) === false ) {
+            violation.url = request.docURL;
+            if ( type === 'script' ) { type = 'inline-script'; }
+            else if ( type === 'font' ) { type = 'inline-font'; }
+        }
+        fctxt.setURL(violation.url)
+             .setType(type)
+             .setFilter(logData)
+             .toLogger();
+    }
+
+    return true;
+};
+
+logCSPViolations.policyDirectiveToTypeMap = new Map([
+    [ 'img-src', 'image' ],
+    [ 'connect-src', 'xmlhttprequest' ],
+    [ 'font-src', 'font' ],
+    [ 'frame-src', 'sub_frame' ],
+    [ 'media-src', 'media' ],
+    [ 'object-src', 'object' ],
+    [ 'script-src', 'script' ],
+    [ 'script-src-attr', 'script' ],
+    [ 'script-src-elem', 'script' ],
+    [ 'style-src', 'stylesheet' ],
+    [ 'style-src-attr', 'stylesheet' ],
+    [ 'style-src-elem', 'stylesheet' ],
+]);
+
+const onMessage = function(request, sender, callback) {
     const tabId = sender && sender.tab ? sender.tab.id : 0;
     const pageStore = µb.pageStoreFromTabId(tabId);
 
@@ -1373,9 +1502,13 @@ var onMessage = function(request, sender, callback) {
     }
 
     // Sync
-    var response;
+    let response;
 
     switch ( request.what ) {
+    case 'applyFilterListSelection':
+        response = µb.applyFilterListSelection(request);
+        break;
+
     case 'domSurveyTransientReport':
         if ( pageStore !== null ) {
             if ( request.filteredElementCount ) {
@@ -1384,11 +1517,11 @@ var onMessage = function(request, sender, callback) {
             if ( request.scriptCount ) {
                 pageStore.scriptCount += request.scriptCount;
             }
-            let broadcastKey = tabId + '-domSurveyReport';
+            const broadcastKey = `${tabId}-domSurveyReport`;
             if ( broadcastTimers.has(broadcastKey) === false ) {
                 broadcastTimers.set(broadcastKey, vAPI.setTimeout(
                     ( ) => { domSurveyFinalReport(tabId); },
-                    53
+                    103
                 ));
             }
         }
@@ -1411,6 +1544,14 @@ var onMessage = function(request, sender, callback) {
         logCosmeticFilters(tabId, request);
         break;
 
+    case 'reloadAllFilters':
+        µb.loadFilterLists();
+        return;
+
+    case 'securityPolicyViolation':
+        response = logCSPViolations(pageStore, request);
+        break;
+
     case 'temporarilyAllowLargeMediaElement':
         if ( pageStore !== null ) {
             pageStore.allowLargeMediaElementsUntil = Date.now() + 2000;
@@ -1430,11 +1571,13 @@ var onMessage = function(request, sender, callback) {
     callback(response);
 };
 
-vAPI.messaging.listen('scriptlets', onMessage);
+vAPI.messaging.listen({
+    name: 'scriptlets',
+    listener: onMessage,
+});
 
-/******************************************************************************/
-
-})();
+// <<<<< end of local scope
+}
 
 
 /******************************************************************************/
