@@ -117,6 +117,7 @@ const fromCosmeticFilter = function(details) {
     const prefix = match[0];
     const exception = prefix.charAt(1) === '@';
     const selector = details.rawFilter.slice(prefix.length);
+    const isHtmlFilter = prefix.endsWith('^');
 
     // The longer the needle, the lower the number of false positives.
     const needle = selector.match(/\w+/g).reduce(function(a, b) {
@@ -182,10 +183,14 @@ const fromCosmeticFilter = function(details) {
                 continue;
             }
 
+            // Do not confuse cosmetic filters with HTML ones.
+            if ( (fargs[0] === 64) !== isHtmlFilter ) { continue; }
+
             switch ( fargs[0] ) {
             // Lowly generic cosmetic filters
             case 0: // simple id-based
                 if (
+                    exception === false &&
                     fargs[1] === selector.slice(1) &&
                     selector.charAt(0) === '#'
                 ) {
@@ -194,6 +199,7 @@ const fromCosmeticFilter = function(details) {
                 break;
             case 2: // simple class-based
                 if (
+                    exception === false &&
                     fargs[1] === selector.slice(1) &&
                     selector.charAt(0) === '.'
                 ) {
@@ -202,7 +208,7 @@ const fromCosmeticFilter = function(details) {
                 break;
             case 1: // complex id-based
             case 3: // complex class-based
-                if ( fargs[2] === selector ) {
+                if ( exception === false && fargs[2] === selector ) {
                     found = prefix + selector;
                 }
                 break;
@@ -216,38 +222,26 @@ const fromCosmeticFilter = function(details) {
                 break;
             // Specific cosmetic filtering
             case 8:
-                if ( exception !== ((fargs[1] & 0b0001) !== 0) ) { break; }
-                isProcedural = (fargs[1] & 0b0010) !== 0;
+            // HTML filtering
+            case 64:
+                if ( exception !== ((fargs[2] & 0b01) !== 0) ) { break; }
+                isProcedural = (fargs[2] & 0b10) !== 0;
                 if (
                     isProcedural === false && fargs[3] !== selector ||
                     isProcedural && JSON.parse(fargs[3]).raw !== selector
                 ) {
                     break;
                 }
-                if ( hostnameMatches(fargs[2]) ) {
-                    found = fargs[2] + prefix + selector;
+                if ( hostnameMatches(fargs[1]) ) {
+                    found = fargs[1] + prefix + selector;
                 }
                 break;
             // Scriptlet injection
             case 32:
-                if ( exception !== ((fargs[1] & 0b0001) !== 0) ) { break; }
+                if ( exception !== ((fargs[2] & 1) !== 0) ) { break; }
                 if ( fargs[3] !== selector ) { break; }
-                if ( hostnameMatches(fargs[2]) ) {
-                    found = fargs[2] + prefix + selector;
-                }
-                break;
-            // HTML filtering
-            case 64: // CSS selector
-            case 65: // procedural
-                if ( exception !== ((fargs[1] & 0b0001) !== 0) ) { break; }
-                if (
-                    fargs[0] === 64 && fargs[3] !== selector ||
-                    fargs[0] === 65 && JSON.parse(fargs[3]).raw !== selector
-                ) {
-                    break;
-                }
-                if ( hostnameMatches(fargs[2]) ) {
-                    found = fargs[2] + prefix + selector;
+                if ( hostnameMatches(fargs[1]) ) {
+                    found = fargs[1] + prefix + selector;
                 }
                 break;
             }
