@@ -65,9 +65,13 @@ var onAllReady = function() {
     initializeTabs();
 
     // https://github.com/chrisaljoudi/uBlock/issues/184
-    // Check for updates not too far in the future.
+    //   Check for updates not too far in the future.
     µb.assets.addObserver(µb.assetObserver.bind(µb));
-    µb.scheduleAssetUpdater(µb.userSettings.autoUpdate ? 5 * 60 * 1000 : 0);
+    µb.scheduleAssetUpdater(
+        µb.userSettings.autoUpdate
+            ? µb.hiddenSettings.autoUpdateDelayAfterLaunch * 1000
+            : 0
+    );
 
     // vAPI.cloud is optional.
     if ( µb.cloudStorageSupported ) {
@@ -173,55 +177,20 @@ const onCommandShortcutsReady = function(commandShortcuts) {
 const onVersionReady = function(lastVersion) {
     if ( lastVersion === vAPI.app.version ) { return; }
 
-    // Since AMO does not allow updating resources.txt, force a reload when a
-    // new version is detected, as resources.txt may have changed since last
-    // release. This will be done only for release versions of Firefox.
-    if (
-        vAPI.webextFlavor.soup.has('firefox') &&
-        vAPI.webextFlavor.soup.has('devbuild') === false
-    ) {
-        µb.redirectEngine.invalidateResourcesSelfie();
-    }
+    // Since built-in resources may have changed since last version, we
+    // force a reload of all resources.
+    µb.redirectEngine.invalidateResourcesSelfie();
 
-    // If unused, just comment out for when we need to compare versions in the
-    // future.
-    const intFromVersion = function(s) {
-        let parts = s.match(/(?:^|\.|b|rc)\d+/g);
-        if ( parts === null ) { return 0; }
-        let vint = 0;
-        for ( let i = 0; i < 4; i++ ) {
-            let pstr = parts[i] || '';
-            let pint;
-            if ( pstr === '' ) {
-                pint = 0;
-            } else if ( pstr.startsWith('.') || pstr.startsWith('b') ) {
-                pint = parseInt(pstr.slice(1), 10);
-            } else if ( pstr.startsWith('rc') ) {
-                pint = parseInt(pstr.slice(2), 10) + 100;
-            } else {
-                pint = parseInt(pstr, 10);
-            }
-            vint = vint * 1000 + pint;
-        }
-        return vint;
-    };
+    const lastVersionInt = vAPI.app.intFromVersion(lastVersion);
 
-    let lastVersionInt = intFromVersion(lastVersion);
-
-    if ( lastVersionInt <= 1016021007 ) {
-        µb.sessionSwitches.toggle('no-scripting', 'behind-the-scene', 2);
-        µb.permanentSwitches.toggle('no-scripting', 'behind-the-scene', 2);
+    // https://github.com/uBlockOrigin/uBlock-issues/issues/494
+    //   Remove useless per-site switches.
+    if ( lastVersionInt <= 1019003007 ) {
+        µb.sessionSwitches.toggle('no-scripting', 'behind-the-scene', 0);
+        µb.permanentSwitches.toggle('no-scripting', 'behind-the-scene', 0);
         µb.saveHostnameSwitches();
     }
 
-    // https://github.com/uBlockOrigin/uBlock-issues/issues/212#issuecomment-419741324
-    if ( lastVersionInt <= 1015024000 ) {
-        if ( µb.hiddenSettings.manualUpdateAssetFetchPeriod === 2000 ) {
-            µb.hiddenSettings.manualUpdateAssetFetchPeriod = 500;
-            µb.saveHiddenSettings();
-        }
-    }
-
     vAPI.storage.set({ version: vAPI.app.version });
 };
 
@@ -233,7 +202,10 @@ const onVersionReady = function(lastVersion) {
 // gorhill 2014-12-15: not anymore
 
 const onNetWhitelistReady = function(netWhitelistRaw) {
-    µb.netWhitelist = µb.whitelistFromString(netWhitelistRaw);
+    if ( typeof netWhitelistRaw === 'string' ) {
+        netWhitelistRaw = netWhitelistRaw.split('\n');
+    }
+    µb.netWhitelist = µb.whitelistFromArray(netWhitelistRaw);
     µb.netWhitelistModifyTime = Date.now();
 };
 
@@ -262,13 +234,6 @@ const onUserSettingsReady = function(fetched) {
     µb.sessionURLFiltering.assign(µb.permanentURLFiltering);
     µb.permanentSwitches.fromString(fetched.hostnameSwitchesString);
     µb.sessionSwitches.assign(µb.permanentSwitches);
-
-    // https://github.com/gorhill/uBlock/issues/1892
-    // For first installation on a battery-powered device, disable generic
-    // cosmetic filtering.
-    if ( µb.firstInstall && vAPI.webextFlavor.soup.has('mobile') ) {
-        userSettings.ignoreGenericCosmeticFilters = true;
-    }
 };
 
 /******************************************************************************/
@@ -318,7 +283,6 @@ const onFirstFetchReady = function(fetched) {
     µb.loadPublicSuffixList().then(( ) => {
         onPSLReady();
     });
-    µb.loadRedirectResources();
 };
 
 /******************************************************************************/
@@ -354,7 +318,6 @@ const createDefaultProps = function() {
         'urlFilteringString': '',
         'hostnameSwitchesString': [
             'no-large-media: behind-the-scene false',
-            'no-scripting: behind-the-scene false'
         ].join('\n'),
         'lastRestoreFile': '',
         'lastRestoreTime': 0,
