@@ -378,40 +378,6 @@
 
 /******************************************************************************/
 
-// I want this helper to be self-maintained, callers must not worry about
-// this helper cleaning after itself by asking them to reset it when it is no
-// longer needed. A timer will be used for self-garbage-collect.
-// Cleaning up 10s after last hit sounds reasonable.
-
-µBlock.stringDeduplicater = {
-    strings: new Map(),
-    timer: undefined,
-    last: 0,
-
-    lookup: function(s) {
-        let t = this.strings.get(s);
-        if ( t === undefined ) {
-            t = this.strings.set(s, s).get(s);
-            if ( this.timer === undefined ) {
-                this.timer = vAPI.setTimeout(() => { this.cleanup(); }, 10000);
-            }
-        }
-        this.last = Date.now();
-        return t;
-    },
-
-    cleanup: function() {
-        if ( (Date.now() - this.last) < 10000 ) {
-            this.timer = vAPI.setTimeout(() => { this.cleanup(); }, 10000);
-        } else {
-            this.timer = undefined;
-            this.strings.clear();
-        }
-    }
-};
-
-/******************************************************************************/
-
 µBlock.openNewTab = function(details) {
     if ( details.url.startsWith('logger-ui.html') ) {
         if ( details.shiftKey ) {
@@ -420,8 +386,8 @@
                 !this.userSettings.alwaysDetachLogger
             );
         }
-        details.popup = this.userSettings.alwaysDetachLogger;
-        if ( details.popup ) {
+        if ( this.userSettings.alwaysDetachLogger ) {
+            details.popup = this.hiddenSettings.loggerPopupType;
             const url = new URL(vAPI.getURL(details.url));
             url.searchParams.set('popup', '1');
             details.url = url.href;
@@ -578,7 +544,7 @@
     }
 
     encode(arrbuf, arrlen) {
-        const inputLength = arrlen >>> 2;
+        const inputLength = (arrlen + 3) >>> 2;
         const inbuf = new Uint32Array(arrbuf, 0, inputLength);
         const outputLength = this.magic.length + 7 + inputLength * 7;
         const outbuf = new Uint8Array(outputLength);
@@ -734,3 +700,15 @@
         return datasetPromise;
     };
 })();
+
+/******************************************************************************/
+
+µBlock.fireDOMEvent = function(name) {
+    if (
+        window instanceof Object &&
+        window.dispatchEvent instanceof Function &&
+        window.CustomEvent instanceof Function
+    ) {
+        window.dispatchEvent(new CustomEvent(name));
+    }
+};
