@@ -13,12 +13,14 @@ cp $SRC/ar/messages.json    $DES/ar/messages.json
 cp $SRC/az/messages.json    $DES/az/messages.json
 cp $SRC/bg/messages.json    $DES/bg/messages.json
 cp $SRC/bn/messages.json    $DES/bn/messages.json
+cp $SRC/bs/messages.json    $DES/bs/messages.json
 cp $SRC/ca/messages.json    $DES/ca/messages.json
 cp $SRC/cs/messages.json    $DES/cs/messages.json
 cp $SRC/cv/messages.json    $DES/cv/messages.json
 cp $SRC/da/messages.json    $DES/da/messages.json
 cp $SRC/de/messages.json    $DES/de/messages.json
 cp $SRC/el/messages.json    $DES/el/messages.json
+cp $SRC/en-GB/messages.json $DES/en_GB/messages.json
 cp $SRC/eo/messages.json    $DES/eo/messages.json
 cp $SRC/es-ES/messages.json $DES/es/messages.json
 cp $SRC/et/messages.json    $DES/et/messages.json
@@ -80,12 +82,14 @@ DES=./dist/description
 cp $SRC/ar/description.txt    $DES/description-ar.txt
 cp $SRC/bg/description.txt    $DES/description-bg.txt
 cp $SRC/bn/description.txt    $DES/description-bn.txt
+cp $SRC/bs/description.txt    $DES/description-bs.txt
 cp $SRC/ca/description.txt    $DES/description-ca.txt
 cp $SRC/cs/description.txt    $DES/description-cs.txt
 cp $SRC/cv/description.txt    $DES/description-cv.txt
 cp $SRC/da/description.txt    $DES/description-da.txt
 cp $SRC/de/description.txt    $DES/description-de.txt
 cp $SRC/el/description.txt    $DES/description-el.txt
+cp $SRC/en-GB/description.txt $DES/description-en_GB.txt
 cp $SRC/eo/description.txt    $DES/description-eo.txt
 cp $SRC/es-ES/description.txt $DES/description-es.txt
 cp $SRC/et/description.txt    $DES/description-et.txt
