@@ -3,24 +3,13 @@
 # This script assumes a linux environment
 
 echo "*** uBlock0.chromium: Creating web store package"
-echo "*** uBlock0.chromium: Copying files"
 
 DES=dist/build/uBlock0.chromium
 rm -rf $DES
 mkdir -p $DES
 
-bash ./tools/make-assets.sh $DES
-
-cp -R src/css               $DES/
-cp -R src/img               $DES/
-cp -R src/js                $DES/
-cp -R src/lib               $DES/
-cp -R src/_locales          $DES/
-cp src/*.html               $DES/
-cp platform/chromium/*.js   $DES/js/
-cp platform/chromium/*.html $DES/
-cp platform/chromium/*.json $DES/
-cp LICENSE.txt              $DES/
+echo "*** uBlock0.chromium: copying common files"
+bash ./tools/copy-common-files.sh  $DES
 
 echo "*** uBlock0.chromium: concatenating content scripts"
 cat $DES/js/vapi-usercss.js > /tmp/contentscript.js
@@ -38,10 +27,6 @@ rm $DES/js/vapi-usercss.pseudo.js
 # Chrome store-specific
 cp -R $DES/_locales/nb $DES/_locales/no
 
-echo "*** uBlock0.chromium: Generating web accessible resources..."
-cp -R src/web_accessible_resources $DES/
-python3 tools/import-war.py $DES/
-
 echo "*** uBlock0.chromium: Generating meta..."
 python tools/make-chromium-meta.py $DES/
 
